"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const fs = require("fs");
const path = require("path");
const opn = require("opn");
const chalk = require("chalk");
const pathExists = require("path-exists");
const node_fetch_1 = require("node-fetch");
const cli_utils_1 = require("@ionic/cli-utils");
const start_1 = require("../lib/start");
const plugins_1 = require("../lib/plugins");
const IONIC_DASH_URL = 'https://apps.ionic.io';
const STARTER_TEMPLATES = [
    {
        name: 'blank',
        description: 'A blank starter project',
        path: 'driftyco/ionic2-starter-blank',
        baseArchive: 'https://github.com/driftyco/ionic2-app-base/archive/master.tar.gz',
        archive: 'https://github.com/driftyco/ionic2-starter-blank/archive/master.tar.gz'
    },
    {
        name: 'tabs',
        description: 'A starting project with a simple tabbed interface',
        path: 'driftyco/ionic2-starter-tabs',
        baseArchive: 'https://github.com/driftyco/ionic2-app-base/archive/master.tar.gz',
        archive: 'https://github.com/driftyco/ionic2-starter-tabs/archive/master.tar.gz'
    },
    {
        name: 'sidemenu',
        description: 'A starting project with a side menu with navigation in the content area',
        path: 'driftyco/ionic2-starter-sidemenu',
        baseArchive: 'https://github.com/driftyco/ionic2-app-base/archive/master.tar.gz',
        archive: 'https://github.com/driftyco/ionic2-starter-sidemenu/archive/master.tar.gz'
    },
    {
        name: 'conference',
        description: 'A project that demonstrates a realworld application',
        path: 'driftyco/ionic-conference-app',
        baseArchive: 'https://github.com/driftyco/ionic2-app-base/archive/master.tar.gz',
        archive: 'https://github.com/driftyco/ionic-conference-app/archive/master.tar.gz'
    },
    {
        name: 'tutorial',
        description: 'A tutorial based project that goes along with the Ionic documentation',
        path: 'driftyco/ionic2-starter-tutorial',
        baseArchive: 'https://github.com/driftyco/ionic2-app-base/archive/master.tar.gz',
        archive: 'https://github.com/driftyco/ionic2-starter-tutorial/archive/master.tar.gz'
    }
];
let StartCommand = class StartCommand extends cli_utils_1.Command {
    run(inputs, options) {
        return __awaiter(this, void 0, void 0, function* () {
            let [projectName, starterTemplateName] = inputs;
            let installer = 'npm';
            let projectRoot;
            if (!start_1.isProjectNameValid(projectName)) {
                throw `Please name your Ionic project something meaningful other than ${chalk.red(projectName)}`;
            }
            projectRoot = path.resolve(projectName);
            projectName = path.basename(projectRoot);
            var tasks = new cli_utils_1.TaskChain();
            /**
             * Create the project directory
             */
            if (!pathExists.sync(projectName)) {
                tasks.next(`Creating directory ${chalk.green(projectRoot)}`);
                fs.mkdirSync(projectRoot);
            }
            else if (!start_1.isSafeToCreateProjectIn(projectRoot)) {
                const response = yield this.env.inquirer.prompt({
                    type: 'confirm',
                    name: 'overwrite',
                    message: `The directory ${chalk.green(projectName)} contains file(s) that could conflict. ` +
                        'Would you like to overwrite the directory with this new project?'
                });
                if (response['overwrite']) {
                    try {
                        tasks.next(`Creating directory ${chalk.green(projectRoot)}`);
                        yield cli_utils_1.rimrafp(projectRoot);
                        fs.mkdirSync(projectRoot);
                    }
                    catch (e) {
                        throw e;
                    }
                }
                else {
                    throw `\nPlease provide a projectName that does not conflict with this directory.\n`;
                }
            }
            let starterTemplate = STARTER_TEMPLATES.find(tpl => tpl['name'] === starterTemplateName);
            if (!starterTemplate) {
                throw `Unable to find starter template for ${starterTemplateName}`;
            }
            /**
             * Download the starter template, gunzip, and untar into the project folder
             */
            tasks.next(`Downloading '${chalk.green(starterTemplateName.toString())}' starter template`);
            let baseArchiveResponse;
            let archiveResponse;
            try {
                [baseArchiveResponse, archiveResponse] = yield Promise.all([
                    node_fetch_1.default(starterTemplate.baseArchive),
                    node_fetch_1.default(starterTemplate.archive)
                ]);
            }
            catch (e) {
                if (['ETIMEOUT', 'ENOTFOUND'].includes(e.code)) {
                    this.env.log.debug(e);
                    this.env.log.error(`Unable to download starter template from github. Please check that you are ` +
                        `able to access the following urls: \n${starterTemplate.baseArchive},\n${starterTemplate.archive}\n`);
                    return;
                }
                throw e;
            }
            if (!baseArchiveResponse || !archiveResponse) {
                this.env.log.error(`Unable to download starter template from github. Please check that you are ` +
                    `able to access the following urls: \n${starterTemplate.baseArchive},\n${starterTemplate.archive}\n`);
                return;
            }
            yield Promise.all([
                start_1.tarXvf(baseArchiveResponse.body, projectRoot),
                start_1.tarXvf(archiveResponse.body, projectRoot)
            ]);
            tasks.next(`Updating project dependencies to add core plugins`);
            const releaseChannelName = yield plugins_1.getReleaseChannelName();
            yield start_1.updateDependenciesForCLI(projectRoot, releaseChannelName);
            /**
             * Download the starter template, gunzip, and untar into the project folder
             */
            if (!options['skip-npm']) {
                tasks.next(`Executing: ${chalk.green(installer + ' install')} within the newly created project directory`);
                if (options['yarn']) {
                    let yarnVersion = yield cli_utils_1.getCommandInfo('yarn', ['-version']);
                    if (yarnVersion) {
                        installer = 'yarn';
                    }
                }
                yield start_1.pkgInstallProject(installer, projectRoot);
            }
            tasks.end();
            /**
             * Print out hello text about how to get started
             */
            this.env.log.msg(start_1.getHelloText());
            /**
             * Ask the user if they would like to create a cloud account
             */
            let { cliFlags } = yield this.env.config.load();
            if (!cliFlags.promptedForSignup) {
                const { createAccount } = yield this.env.inquirer.prompt({
                    type: 'confirm',
                    name: 'createAccount',
                    message: 'Create a free Ionic account to add features like User Authentication, ' +
                        'Push Notifications, Live Updating, iOS builds, and more?'
                });
                if (createAccount) {
                    opn(IONIC_DASH_URL + '/signup', { wait: false });
                }
                cliFlags.promptedForSignup = true;
                this.env.log.msg(`\n`);
            }
            if (!cliFlags.promptedForTelemetry) {
                const { optIn } = yield this.env.inquirer.prompt({
                    type: 'confirm',
                    name: 'optIn',
                    message: 'Would you like to help Ionic improve the CLI by providing anonymous ' +
                        'usage and error reporting information?'
                });
                cliFlags.promptedForTelemetry = true;
                cliFlags.enableTelemetry = optIn;
            }
            this.env.log.msg(`\nGo to your newly created project: ${chalk.green(`cd ${projectRoot}`)}\n`);
        });
    }
};
StartCommand = __decorate([
    cli_utils_1.CommandMetadata({
        name: 'start',
        description: 'Starts a new Ionic project in a new directory.',
        exampleCommands: ['mynewapp blank'],
        inputs: [
            {
                name: 'name',
                description: 'directory and name for the new project',
                validators: [cli_utils_1.validators.required],
                prompt: {
                    message: 'What would you like to name your project:'
                }
            },
            {
                name: 'template',
                description: `Starter templates can either come from a named template (ex: ${STARTER_TEMPLATES.map(st => st.name).join(', ')})`,
                validators: [cli_utils_1.validators.required],
                prompt: {
                    type: 'list',
                    message: 'What starter would you like to use',
                    choices: start_1.getStarterTemplateTextList(STARTER_TEMPLATES).map((text, i) => ({
                        name: text,
                        short: STARTER_TEMPLATES[i].name,
                        value: STARTER_TEMPLATES[i].name
                    }))
                }
            }
        ],
        options: [
            {
                name: 'appname',
                description: 'Human readable name for the app (Use quotes around the name',
                aliases: ['a']
            },
            {
                name: 'id',
                description: 'Package name for <widget id> config, ex: com.mycompany.myapp',
                aliases: ['i']
            },
            {
                name: 'skip-npm',
                description: 'Skip npm package installation',
                type: Boolean,
                aliases: []
            },
            {
                name: 'no-cordova',
                description: 'Create a basic structure without Cordova requirements',
                type: Boolean,
                aliases: ['w']
            },
            {
                name: 'list',
                description: 'List starter templates available',
                type: Boolean,
                aliases: ['l']
            },
            {
                name: 'cloud-app-id',
                description: 'An existing Ionic.io app ID to link with',
                aliases: []
            }
        ]
    })
], StartCommand);
exports.StartCommand = StartCommand;
