"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const minimist = require("minimist");
const path = require("path");
const chalk = require("chalk");
const cli_utils_1 = require("@ionic/cli-utils");
const plugins_1 = require("./lib/plugins");
const PROJECT_FILE = 'ionic.config.json';
function cleanup() {
    for (let task of cli_utils_1.TASKS) {
        if (task.running) {
            task.fail();
        }
        task.clear();
    }
}
function run(pargv, env) {
    return __awaiter(this, void 0, void 0, function* () {
        let exitCode = 0;
        let err;
        pargv = modifyArguments(pargv.slice(2));
        const argv = minimist(pargv);
        const log = new cli_utils_1.Logger();
        if (argv['log-level']) {
            log.level = argv['log-level'];
        }
        // If an legacy command is being executed inform the user that there is a new command available
        let foundCommand = mapLegacyCommand(argv._[0]);
        if (foundCommand) {
            log.msg(`The ${chalk.bold(argv._[0])} command is no longer available. To find out more about the equivalent please run:\n\n` +
                `  ${chalk.green(`ionic ${foundCommand} --help`)}\n`);
            return;
        }
        env['PROJECT_FILE'] = PROJECT_FILE;
        env['PROJECT_DIR'] = yield getProjectRootDir(process.cwd(), env['PROJECT_FILE']);
        try {
            const [plugin, inputs] = yield plugins_1.resolvePlugin(env['PROJECT_DIR'], pargv);
            yield plugin.run(inputs, env);
        }
        catch (e) {
            err = e;
        }
        cleanup();
        if (err) {
            exitCode = 1;
            if (cli_utils_1.isSuperAgentError(err)) {
                console.error(cli_utils_1.formatError(err));
            }
            else if (err instanceof cli_utils_1.FatalException) {
                exitCode = err.exitCode;
                if (err.message) {
                    log.error(err.message);
                }
            }
            else {
                console.error(err);
            }
            process.exit(exitCode);
        }
    });
}
exports.run = run;
/**
 * Find the base project directory based on the dir input
 */
function getProjectRootDir(dir, projectFileName) {
    return __awaiter(this, void 0, void 0, function* () {
        const dirInfo = path.parse(dir);
        const directoriesToCheck = dirInfo.dir
            .slice(dirInfo.root.length)
            .split(path.sep)
            .concat(dirInfo.base)
            .map((segment, index, array) => {
            let pathSegments = array.slice(0, (array.length - index));
            return dirInfo.root + path.join(...pathSegments);
        });
        for (let i = 0; i < directoriesToCheck.length; i++) {
            const results = yield cli_utils_1.fsReadDir(directoriesToCheck[i]);
            if (results.includes(projectFileName)) {
                return directoriesToCheck[i];
            }
        }
        return '';
    });
}
/**
 * Map legacy options to their new equivalent
 */
function modifyArguments(pargv) {
    let modifiedArgArray = pargv.slice();
    const minimistArgv = minimist(pargv);
    /**
     * Replace command to be executed
     */
    if (pargv.length === 0) {
        return ['help'];
    }
    if (minimistArgv['help'] || minimistArgv['h']) {
        if (minimistArgv._.length > 0) {
            return ['help', minimistArgv._[0]];
        }
        else {
            return ['help'];
        }
    }
    if (minimistArgv._.length === 0 && (minimistArgv['version'] || minimistArgv['v'])) {
        return modifiedArgArray = ['version'];
    }
    /**
     * Change command executed
     */
    if (minimistArgv._[0] === 'lab') {
        modifiedArgArray[0] = 'serve';
        modifiedArgArray.push('--lab');
    }
    /**
     * Change command options
     */
    if (minimistArgv['verbose']) {
        modifiedArgArray[modifiedArgArray.indexOf('--verbose')] = '--log-level="debug"';
    }
    return modifiedArgArray;
}
/**
 * Find the command that is the equivalent of a legacy command.
 */
function mapLegacyCommand(command) {
    const commandMap = {
        'build': 'cordova:build',
        'compile': 'cordova:compile',
        'emulate': 'cordova:emulate',
        'platform': 'cordova:platform',
        'prepare': 'cordova:prepare',
        'resources': 'cordova:resources',
        'run': 'cordova:run',
    };
    return commandMap[command];
}
